
/******************************************************************************
	 *  Compilation:  javac EchoServer.java
	 *  Execution:    java EchoServer port
	 *  Dependencies: In.java Out.java
	 *  
	 *  Runs an echo server which listents for connections on port 4444,
	 *  and echoes back whatever is sent to it.
	 *
	 *
	 *  % java EchoServer 4444
	 *
	 *
	 *  Limitations
	 *  -----------
	 *  The server is not multi-threaded, so at most one client can connect
	 *  at a time.
	 *
	 ******************************************************************************/

import java.net.Socket;

import org.apache.commons.io.IOUtils;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.ServerSocket;

public class EchoServer {

	public static void main(String[] args) throws Exception {

		// create socket
		int port = 4444;
		ServerSocket serverSocket = new ServerSocket(port);
		System.err.println("Started server on port " + port);

		// repeatedly wait for connections, and process
		while (true) {

			// a "blocking" call which waits until a connection is requested
			Socket clientSocket = serverSocket.accept();
			System.err.println("Accepted connection from client");

			// open up IO streams
			InputStream in = null;
			OutputStream out = null;
			try {
				in = new BufferedInputStream(clientSocket.getInputStream());
				out = new BufferedOutputStream(clientSocket.getOutputStream());

				// waits for data and reads it in until connection dies
				// readLine() blocks until the server receives a new line from
				// client
				String s = IOUtils.toString(in, "UTF-8");
				System.out.println("*** received ***");
				System.out.println(s);
				System.out.println("*** end ***");
				IOUtils.write(s, out, "UTF-8");
			} catch (Exception e) {
				e.printStackTrace();
			} finally {
				// close IO streams, then socket
				System.err.println("Closing connection with client");
				out.close();
				in.close();
				clientSocket.close();
			}
		}
	}
}
